@@ -20,7 +20,7 @@ use crate::{
     },
     pipelines::rust::sri::{SriBuilder, SriOptions, SriType},
     processing::{integrity::IntegrityType, minify::minify_js},
-    tools::{self, Application},
+    tools::{self, Application, ToolInformation},
 };
 use anyhow::{anyhow, bail, ensure, Context, Result};
 use cargo_metadata::Artifact;
@@ -36,7 +36,7 @@ use std::{
 };
 use tokio::{fs, io::AsyncWriteExt, process::Command, sync::mpsc, task::JoinHandle};
 use tracing::log;
-use wasm_bindgen::{find_wasm_bindgen_version, WasmBindgenTarget};
+use wasm_bindgen::{find_wasm_bindgen_version, WasmBindgenFeatures, WasmBindgenTarget};
 use wasm_opt::WasmOptLevel;
 
 /// A Rust application pipeline.
@@ -78,6 +78,8 @@ pub struct RustApp {
     /// An optional optimization setting that enables wasm-opt. Can be nothing, `0` (default), `1`,
     /// `2`, `3`, `4`, `s or `z`. Using `0` disables wasm-opt completely.
     wasm_opt: WasmOptLevel,
+    /// An optional optimization command line params to wasm-opt if it is enabled.
+    wasm_opt_params: Vec<String>,
     /// The value of the `--target` flag for wasm-bindgen.
     wasm_bindgen_target: WasmBindgenTarget,
     /// Name for the module. Is binary name if given, otherwise it is the name of the cargo
@@ -169,6 +171,12 @@ impl RustApp {
                     WasmOptLevel::Off
                 }
             });
+        let wasm_opt_params = attrs
+            .get("data-wasm-opt-params")
+            .iter()
+            .flat_map(|val| val.split_whitespace())
+            .map(|val| val.to_string())
+            .collect();
         let wasm_bindgen_target = attrs
             .get("data-bindgen-target")
             .map(|s| s.parse())
@@ -282,6 +290,7 @@ impl RustApp {
             reference_types,
             weak_refs,
             wasm_opt,
+            wasm_opt_params,
             wasm_bindgen_target,
             app_type,
             name,
@@ -331,6 +340,7 @@ impl RustApp {
             reference_types: false,
             weak_refs: false,
             wasm_opt: WasmOptLevel::Off,
+            wasm_opt_params: Default::default(),
             app_type: RustAppType::Main,
             wasm_bindgen_target: WasmBindgenTarget::Web,
             name,
@@ -513,13 +523,17 @@ impl RustApp {
     #[tracing::instrument(level = "trace", skip(self))]
     async fn wasm_bindgen_build(&mut self, wasm_path: &Path) -> Result<RustAppOutput> {
         let version = find_wasm_bindgen_version(&self.cfg.tools, &self.manifest);
-        let wasm_bindgen = tools::get(
+        let ToolInformation {
+            path: wasm_bindgen,
+            version,
+        } = tools::get_info(
             Application::WasmBindgen,
             version.as_deref(),
             self.cfg.offline,
             &self.cfg.client_options(),
         )
         .await?;
+        let wasm_bindgen_features = WasmBindgenFeatures::from_version(&version)?;
 
         // Ensure our output dir is in place.
         let wasm_bindgen_name = Application::WasmBindgen.name();
@@ -737,6 +751,7 @@ impl RustApp {
             import_bindings: self.import_bindings,
             import_bindings_name: self.import_bindings_name.clone(),
             initializer,
+            wasm_bindgen_features,
         })
     }
 
@@ -890,6 +905,7 @@ impl RustApp {
         let output = output.join(format!("{}_bg.wasm", self.name));
         let arg_output = format!("--output={output}");
         let arg_opt_level = format!("-O{}", self.wasm_opt.as_ref());
+        let arg_opt_params = self.wasm_opt_params.as_slice();
         let target_wasm = self
             .cfg
             .staging_dist
@@ -902,6 +918,8 @@ impl RustApp {
             args.push("--enable-reference-types");
         }
 
+        args.extend(arg_opt_params.iter().map(|s| s.as_str()));
+
         // Invoke wasm-opt.
         tracing::debug!("calling wasm-opt");
         common::run_command(wasm_opt_name, &wasm_opt, &args, &self.cfg.working_directory)
